/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 *
 * Version: 5.2.0 (TBD)
 */
(function () {
    'use strict';

    var global = tinymce.util.Tools.resolve('tinymce.PluginManager');

    var init = function (editor, data) {
      var config = data.config;
      var options = {
        ch: '@',
        columns: 1,
        minChars: config.minChars || 1,
        maxResults: config.maxResults || 10,
        highlightOn: config.highlightOn,
        matches: function (rng, text, pattern) {
          if (config.matches != null) {
            return config.matches(rng, text, pattern);
          }
          return true;
        },
        fetch: function (pattern, maxResults) {
          if (config.fetch == null) {
            throw new Error('fetch is required.');
          }
          return config.fetch(editor, pattern, maxResults);
        },
        onAction: function (autocompleteApi, rng, value, meta) {
          if (config.onAction) {
            config.onAction(editor, autocompleteApi, rng, value, meta);
          } else {
            editor.selection.setRng(rng);
            editor.insertContent('@' + value);
            autocompleteApi.hide();
          }
        }
      };
      editor.ui.registry.addAutocompleter('acex', options);
    };

    var getAcexConfig = function (editor) {
      return editor.getParam('acex', {}, 'object');
    };
    var Settings = { getAcexConfig: getAcexConfig };

    function Plugin () {
      global.add('acex', function (editor, pluginUrl) {
        var data = {
          config: Settings.getAcexConfig(editor),
          pluginUrl: pluginUrl
        };
        init(editor, data);
      });
    }

    Plugin();

})();
