/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 *
 * Version: 5.2.0 (TBD)
 */
(function () {
    'use strict';

    var global$1 = tinymce.util.Tools.resolve('tinymce.PluginManager');

    var global = tinymce.util.Tools.resolve('tinymce.Env');

    var getAutoLinkPattern = function (editor) {
      return editor.getParam('autolink_pattern', /^(https?:\/\/|ssh:\/\/|ftp:\/\/|file:\/|www\.|(?:mailto:)?[A-Z0-9._%+\-]+@)(.+)$/i);
    };
    var getDefaultLinkTarget = function (editor) {
      return editor.getParam('default_link_target', false);
    };
    var getDefaultLinkProtocol = function (editor) {
      return editor.getParam('link_default_protocol', 'http', 'string');
    };
    var Settings = {
      getAutoLinkPattern: getAutoLinkPattern,
      getDefaultLinkTarget: getDefaultLinkTarget,
      getDefaultLinkProtocol: getDefaultLinkProtocol
    };

    var rangeEqualsDelimiterOrSpace = function (rangeString, delimiter) {
      return rangeString === delimiter || rangeString === ' ' || rangeString.charCodeAt(0) === 160;
    };
    var handleEclipse = function (editor) {
      parseCurrentLine(editor, -1, '(');
      applyCurrentLine();
    };
    var handleSpacebar = function (editor) {
      parseCurrentLine(editor, 0, '');
      applyCurrentLine();
    };
    var handleEnter = function (editor) {
      parseCurrentLine(editor, -1, '');
    };
    var handleLine = function (editor, endOffset, delimiter) {
      parseCurrentLine(editor, endOffset, delimiter);
      applyCurrentLine();
    };
    var scopeIndex = function (container, index) {
      if (index < 0) {
        index = 0;
      }
      if (container.nodeType === 3) {
        var len = container.data.length;
        if (index > len) {
          index = len;
        }
      }
      return index;
    };
    var setStart = function (rng, container, offset) {
      if (container.nodeType !== 1 || container.hasChildNodes()) {
        rng.setStart(container, scopeIndex(container, offset));
      } else {
        rng.setStartBefore(container);
      }
    };
    var setEnd = function (rng, container, offset) {
      if (container.nodeType !== 1 || container.hasChildNodes()) {
        rng.setEnd(container, scopeIndex(container, offset));
      } else {
        rng.setEndAfter(container);
      }
    };
    var lineInfo = null;
    var parseCurrentLine = function (editor, endOffset, delimiter) {
      var rng, end, start, endContainer, text, matches, prev, len, rngText;
      if (editor.selection.getNode().tagName === 'A') {
        return;
      }
      rng = editor.selection.getRng(true).cloneRange();
      var offsetLimit = 5;
      var prevTag = rng.startContainer != null ? rng.startContainer.previousSibling : null;
      if (prevTag != null && prevTag.nodeType === 1 && prevTag.tagName === 'A') {
        offsetLimit = 2;
      }
      if (rng.startOffset < offsetLimit) {
        prev = rng.endContainer.previousSibling;
        if (!prev) {
          if (!rng.endContainer.firstChild || !rng.endContainer.firstChild.nextSibling) {
            return;
          }
          prev = rng.endContainer.firstChild.nextSibling;
        }
        len = prev.length;
        setStart(rng, prev, len);
        setEnd(rng, prev, len);
        if (rng.endOffset < offsetLimit) {
          return;
        }
        end = rng.endOffset;
        endContainer = prev;
      } else {
        endContainer = rng.endContainer;
        if (endContainer.nodeType !== 3 && endContainer.firstChild) {
          while (endContainer.nodeType !== 3 && endContainer.firstChild) {
            endContainer = endContainer.firstChild;
          }
          if (endContainer.nodeType === 3) {
            setStart(rng, endContainer, 0);
            setEnd(rng, endContainer, endContainer.nodeValue.length);
          }
        }
        if (rng.endOffset === 1) {
          end = 2;
        } else {
          end = rng.endOffset - 1 - endOffset;
        }
      }
      start = end;
      var numCheck = true;
      var isTel = false;
      var subNumCheck = true;
      var telRegex = /^(tel:)?[ \u00A0]?([+]?)([(]{0,1}[0-9]{1,4}[)]{0,1}[- \u00A0\t\./0-9]*[0-9]+[.]?)[ \u00A0]?((;|,+|ext)([#0-9]+))?$/i;
      var telCharRegex = /[ \u00A0tel:\d().+\-,;#x]/i;
      var subTelRegex = /[ \u00A0\d.\-,;#ext]/i;
      var telOutRegex = /[ \u00A0]{2,}|[-.]{2,}|([-.][ \u00A0][-.])/;
      var firstSpacePos = -1;
      var lastSpacePos = -1;
      var prevLastSpacePos = -1;
      var prevATag = null;
      var curStr = '';
      do {
        setStart(rng, endContainer, end >= 2 ? end - 2 : 0);
        setEnd(rng, endContainer, end >= 1 ? end - 1 : 0);
        end -= 1;
        rngText = rng.toString();
        curStr = rngText + curStr;
        if (rngText === ' ' || end - 2 < 0) {
          lastSpacePos = end;
          if (firstSpacePos === -1) {
            firstSpacePos = end;
          }
          if (subNumCheck === true) {
            var prev_1 = endContainer.previousSibling;
            if (prev_1 != null && prev_1.nodeType === 1 && prev_1.tagName === 'A' && telRegex.test(prev_1.textContent) === true) {
              isTel = true;
              prevATag = prev_1;
            }
          } else if (numCheck === true) {
            var tmpCheck = curStr.match(/([tel:]+)/gi);
            if (tmpCheck != null && (tmpCheck.length > 1 || tmpCheck[0].toLowerCase().indexOf('tel:') !== 0)) {
              numCheck = false;
              lastSpacePos = prevLastSpacePos;
              if (lastSpacePos !== -1) {
                isTel = true;
                if (lastSpacePos > 0) {
                  setStart(rng, endContainer, lastSpacePos - 1);
                  setEnd(rng, endContainer, lastSpacePos);
                }
              }
            }
          }
          prevLastSpacePos = lastSpacePos;
        }
        if (numCheck === true && (telCharRegex.test(rngText) !== true || end - 2 < 0)) {
          numCheck = false;
          if (lastSpacePos !== -1) {
            isTel = true;
          }
        }
        if (subNumCheck === true && (subTelRegex.test(rngText) !== true || end - 2 < 0)) {
          subNumCheck = false;
        }
      } while ((firstSpacePos === -1 || subNumCheck === true || numCheck === true) && rngText !== '' && rngText.charCodeAt(0) !== 160 && end - 2 >= 0 && rngText !== delimiter);
      if (isTel === false) {
        end = firstSpacePos;
      } else {
        end = lastSpacePos;
      }
      if (prevATag != null) {
        setStart(rng, prevATag, 0);
        setEnd(rng, endContainer, start);
        document.execCommand('unlink');
      } else if (rangeEqualsDelimiterOrSpace(rng.toString(), delimiter)) {
        setStart(rng, endContainer, end);
        end += 1;
        setEnd(rng, endContainer, start);
      } else if (rng.startOffset === 0) {
        setStart(rng, endContainer, 0);
        setEnd(rng, endContainer, start);
      } else {
        setStart(rng, endContainer, end);
        setEnd(rng, endContainer, start);
      }
      text = rng.toString();
      if (text.charAt(text.length - 1) === '.') {
        setEnd(rng, endContainer, start - 1);
      }
      text = rng.toString().trim();
      var linkURL = null;
      if (isTel === true) {
        if (telOutRegex.test(text) === false && text.length < 35) {
          matches = text.match(telRegex);
          var isAllNumber = /^[0-9.]{4,}$/.test(text) === true;
          if (isAllNumber === true) {
            var pcnt = (text.match(/[.]/g) || []).length;
            if (pcnt > 1) {
              isAllNumber = false;
            }
          }
          if (matches && isAllNumber === false) {
            var telNum = matches[3].replace(/[ ().+-]/g, '');
            if (matches[2] !== '+' && telNum.length >= 6 || matches[2] === '+' && telNum.length >= 7) {
              if (matches[2] === '+') {
                while (telNum.indexOf('0') === 0) {
                  telNum = telNum.substring(1);
                }
              }
              if (telNum.length < 26) {
                linkURL = 'tel:' + matches[2] + telNum;
              }
              if (matches[5] != null && matches[6] != null) {
                if (matches[5] === 'ext') {
                  linkURL += ';' + matches[6];
                } else {
                  linkURL += matches[5] + matches[6];
                }
              }
            }
          }
        }
      } else {
        if (text.indexOf('@') !== -1) {
          var emailRegex = /^(mailto:)?([^\s@:]+@[^\s@]+\.[^\s@]{2,})$/i;
          matches = text.match(emailRegex);
          if (matches) {
            if (matches[1] == null) {
              linkURL = 'mailto:' + matches[2];
            } else {
              linkURL = matches[0];
            }
          }
        } else {
          var protocol = Settings.getDefaultLinkProtocol(editor);
          var urlRegex = /^((https?:\/\/|ssh:\/\/|ftp:\/\/|file:\/){1}|(www\.){1})([-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b){1}((\/{1}[-a-zA-Z0-9()=._]*)*)(#|\?|$){1}([-a-zA-Z0-9()@:%_\+.\?#,~&//=]*)$/i;
          matches = text.match(urlRegex);
          if (matches) {
            if (matches[2] == null) {
              linkURL = protocol + '://' + matches[0];
            } else {
              linkURL = matches[0];
            }
          }
        }
      }
      if (linkURL !== null) {
        lineInfo = {
          editor: editor,
          link: linkURL,
          range: {
            startContainer: rng.startContainer,
            startOffset: rng.startOffset,
            endContainer: rng.endContainer,
            endOffset: rng.endOffset
          }
        };
      } else {
        lineInfo = null;
      }
      return lineInfo !== null;
    };
    var applyCurrentLine = function () {
      var editor, linkURL, rng, bookmark;
      if (lineInfo != null) {
        editor = lineInfo.editor;
        linkURL = lineInfo.link;
        rng = editor.selection.getRng(true).cloneRange();
        var range = lineInfo.range;
        setStart(rng, range.startContainer, range.startOffset);
        setEnd(rng, range.endContainer, range.endOffset);
        lineInfo = null;
      }
      if (linkURL != null) {
        editor.undoManager.transact(function () {
          bookmark = editor.selection.getBookmark();
          editor.selection.setRng(rng);
          editor.execCommand('createlink', false, linkURL);
          var defaultLinkTarget = Settings.getDefaultLinkTarget(editor);
          if (defaultLinkTarget !== false) {
            editor.dom.setAttrib(editor.selection.getNode(), 'target', defaultLinkTarget);
          }
          editor.selection.moveToBookmark(bookmark);
          editor.nodeChanged();
        });
      }
    };
    var isEnabled = true;
    var setup = function (editor) {
      var autoUrlDetectState;
      editor.on('keydown', function (e) {
        if (e.keyCode === 13 && isEnabled === true) {
          handleEnter(editor);
        }
      });
      if (global.browser.isIE()) {
        editor.on('focus', function () {
          if (!autoUrlDetectState) {
            autoUrlDetectState = true;
            try {
              editor.execCommand('AutoUrlDetect', false, true);
            } catch (ex) {
            }
          }
        });
        return;
      }
      editor.on('keypress', function (e) {
        if (e.keyCode === 41 && isEnabled === true) {
          handleEclipse(editor);
          applyCurrentLine();
        }
      });
      editor.on('keyup', function (e) {
        if (e.keyCode === 32 && isEnabled === true) {
          handleSpacebar(editor);
        } else if (e.keyCode === 13 && isEnabled === true) {
          applyCurrentLine();
        }
      });
    };
    var setEnabled = function (val) {
      isEnabled = val;
    };
    var Keys = {
      setup: setup,
      apis: {
        setEnabled: setEnabled,
        handleEnter: handleEnter,
        handleSpacebar: handleSpacebar,
        handleLine: handleLine
      }
    };

    function Plugin () {
      global$1.add('autolink', function (editor) {
        Keys.setup(editor);
        return Keys.apis;
      });
    }

    Plugin();

})();
